mod p2000;

use std::{fs, io::Write, path::PathBuf, str::FromStr, sync::Arc};

use eventsource_client::Client;
use p2000::Message;
use reqwest::multipart::{Form, Part};
use serde::Deserialize;
use serde_json::Value;
use serenity::{
    all::{Color, Timestamp},
    builder::{CreateEmbed, CreateMessage},
    futures::TryStreamExt,
};

type Error = Box<dyn std::error::Error + Send + Sync>;
type Result<T> = std::result::Result<T, Error>;

static COLOR_POLITIE: Color = Color::from_rgb(0, 70, 130);
static COLOR_BRANDWEER: Color = Color::from_rgb(208, 2, 27);
static COLOR_AMBULANCE: Color = Color::from_rgb(241, 221, 56);

static LOGO_POLITIE: &[u8; 10906] = include_bytes!("../res/politie.png");
static LOGO_BRANDWEER: &[u8; 18092] = include_bytes!("../res/brandweer.png");
static LOGO_AMBULANCE: &[u8; 15241] = include_bytes!("../res/ambulance.png");

#[derive(Deserialize, Clone)]
struct Config {
    webhook_url: String,
    keywords: Vec<String>,
}

#[tokio::main]
async fn main() -> Result<()> {
    dotenvy::dotenv().ok();

    let mut config_dir = match dirs::config_dir() {
        Some(mut dir) => {
            dir.push("p2000");
            dir
        }
        None => PathBuf::from_str(".")?,
    };

    // create if not exists
    if let Err(e) = std::fs::create_dir_all(&config_dir) {
        println!("Failed to create directory: {:?} - {:?}", &config_dir, e);
    }

    config_dir.push("config.json");

    // create if not exists config file "[]"
    if !fs::exists(&config_dir)? {
        let mut file = fs::File::create(&config_dir)?;
        write!(file, "[]")?;
    }

    // load config file
    let config = std::fs::read_to_string(&config_dir)?;
    let config: Vec<Config> = serde_json::from_str(&config)?;

    if config.is_empty() {
        return Err(format!(
            "No webhooks configured in {}",
            &config_dir.to_string_lossy()
        )
        .into());
    }

    let es = eventsource_client::ClientBuilder::for_url("https://p2000.avii.nl/sse")?.build();
    let client = es.stream();
    let mut stream = client
        .map_ok(|event| {
            if let eventsource_client::SSE::Event(ev) = event {
                return Some(ev.data);
            }
            None
        })
        .map_err(|err| eprintln!("error streaming events: {:?}", err));

    while let Ok(Some(buffer)) = stream.try_next().await {
        let Some(buffer) = buffer else {
            continue;
        };
        let Ok(message) = Message::from_str(&buffer) else {
            continue;
        };
        let msg_str = message.to_string();
        println!("{}", msg_str);

        let message = Arc::new(message);

        for c in config.iter() {
            if c.keywords.is_empty() || c.keywords.iter().any(|keyword| msg_str.contains(keyword)) {
                let webook_url = c.webhook_url.to_string();
                let message = message.clone();
                tokio::spawn(async move {
                    let embed = match create_embed(message) {
                        Ok(embed) => embed,
                        Err(e) => {
                            eprintln!("Failed to create embed: {:?}", e);
                            return;
                        }
                    };

                    if let Err(e) = send(webook_url, embed).await {
                        eprintln!("Error sending message: {:?}", e);
                    }
                });
            }
        }
    }

    Ok(())
}

async fn send(url: String, body: Form) -> Result<()> {
    let client = reqwest::ClientBuilder::new().use_rustls_tls().build()?;
    client.post(url).multipart(body).send().await?;
    Ok(())
}

fn create_embed(message: Arc<Message>) -> Result<Form> {
    let mut embeds = vec![];

    let mut builder = CreateMessage::new();
    let mut form = reqwest::multipart::Form::new();
    let mut filenum = 1;

    let mut has_politie = false;
    let mut has_brandweer = false;
    let mut has_ambulance = false;

    for capcode in message.capcodes.iter() {
        let color = match capcode.service {
            p2000::Service::Politie => COLOR_POLITIE,
            p2000::Service::Brandweer => COLOR_BRANDWEER,
            p2000::Service::Ambulance => COLOR_AMBULANCE,
            p2000::Service::Overige(_) => Color::from_rgb(127, 127, 127),
        };

        let mut embed = CreateEmbed::new()
            .color(color)
            .title(capcode.service.to_string())
            .description(capcode.capcode)
            .field(message.message.clone(), "", false)
            .field("", capcode.region, true)
            .field("", capcode.location, true)
            .field("", capcode.description, true);

        match capcode.service {
            p2000::Service::Politie => {
                if !has_politie {
                    has_politie = true;
                    form = form.part(
                        format!("file{}", filenum),
                        Part::bytes(LOGO_POLITIE).file_name("politie.png"),
                    );
                    filenum += 1;
                }
                embed = embed.thumbnail("attachment://politie.png");
            }
            p2000::Service::Brandweer => {
                if !has_brandweer {
                    has_brandweer = true;
                    form = form.part(
                        format!("file{}", filenum),
                        Part::bytes(LOGO_BRANDWEER).file_name("brandweer.png"),
                    );
                    filenum += 1;
                }
                embed = embed.thumbnail("attachment://brandweer.png");
            }
            p2000::Service::Ambulance => {
                if !has_ambulance {
                    has_ambulance = true;
                    form = form.part(
                        format!("file{}", filenum),
                        Part::bytes(LOGO_AMBULANCE).file_name("ambulance.png"),
                    );
                    filenum += 1;
                }
                embed = embed.thumbnail("attachment://ambulance.png");
            }
            p2000::Service::Overige(_) => (),
        };

        embed = embed.timestamp(Timestamp::now());
        embeds.push(embed);
    }

    if embeds.is_empty() {
        let mut embed = CreateEmbed::new().color(Color::from_rgb(127, 127, 127));
        embed = embed.field(message.message.clone(), "", false);

        for code in message.capcodes_raw.split_whitespace() {
            embed = embed.field("", code, true);
        }
        embed = embed.timestamp(Timestamp::now());
        embeds.push(embed);
    }

    builder = builder.add_embeds(embeds);

    let json_data = serde_json::to_string_pretty(&builder)?;
    let mut data = serde_json::from_str::<Value>(&json_data)?;

    data["username"] = serde_json::Value::String("p2000".to_string());
    data["avatar_url"] =
        serde_json::Value::String("https://public.avii.nl/.images/p2000_3.png".to_string());

    let json_data = serde_json::to_string_pretty(&data)?;
    form = form.text("payload_json", json_data);

    Ok(form)
}
